#!/usr/bin/env ruby
#
# LaunchBar Action Script
#
require 'json'
require 'date'
require 'shellwords'

# DateTime methods
class DateTime
  def today?
    now = Time.now
    today = now.strftime('%Y%m%d')

    strftime('%Y%m%d') == today
  end
end

def doing_show(section, limit = 9)
  items = []
  output = `/usr/local/bin/doing show #{section} -c #{limit} -s desc -o json`
  data = JSON.parse(output)
  data['items'].each do |entry|
    item = {}
    date = DateTime.parse(entry['date'])
    formatted = date.today? ? date.strftime('%H:%M') : date.strftime('%m/%d')

    icon = 'font-awesome:hourglass-start'
    title = entry['title']
    if title =~ /@done/
      icon = 'font-awesome:check-square'
      title.sub!(/@done(\(.*?\))?/, '')
      # title.gsub!(/@(\S+)/,'\1')
    end
    item['title'] = "#{formatted}: #{entry['title']}"
    item['subtitle'] = entry['note']
    item['badge'] = entry['time']
    # item['label'] = entry['tags'].map {|t| "@#{t}" }.join(", ")
    item['icon'] = icon
    item['alwaysShowsSubtitle'] = true
    items.push(item)
  end

  timers = []
  data['timers'].each do |timer|
    item = {}
    item['title'] = "@#{timer['tag']}"
    item['badge'] = timer['formatted']
    item['icon'] = 'font-awesome:hourglass-end'
    timers.push(item)
  end

  items.push({ 'title' => 'Tags and Timers', 'children' => timers })

  items
end

def doing_parse(doing)
  items = []

  return doing_show(doing.sub(/^show\s*/, ''), 0) if doing =~ /^show\b/

  cmd = ['/usr/local/bin/doing --stdout']

  args = []

  doing.gsub!(/(?<= |^)(-(b|t|s)|--(back|took|section))(?!==)( +\S+)/) do |m|
    args.push(m)
    ''
  end

  doing.gsub!(/(?<= |^)--(back|took|section)=(\S+)/) do
    m = Regexp.last_match
    args.push("--#{m[1]} #{m[2]}")
    ''
  end

  doing.sub!(/\(.*?\)/) do |m|
    args.push("-n #{Shellwords.escape(m)}")
    ''
  end

  doing.gsub!(/ +/, ' ')
  doing.strip!

  case doing.strip
  when /^done/i
    cmd.push('done')
    text = doing.sub(/^done/i, '').strip
    cmd.push(text) if text.length.positive?
  when /^finish$/i
    cmd.push(doing)
    text = doing.sub(/^finish/, '').strip
    cmd.push(text) if text.length.positive?
  when /^meanwhile/i
    cmd.push('meanwhile')
    text = doing.sub(/^meanwhile/, '').strip
    cmd.push(text) if text.length.positive?
  when /^(again|resume)$/i
    cmd.push('again')
  else
    args.push('-f')
    cmd.push('now')
    text = doing.sub(/^now/i, '').strip
    cmd.push(text) if text.length.positive?
  end
  command = "#{cmd.join(' ')} #{args.join(' ')}".strip
  result = `#{command}`

  item = {}
  item['title'] = result
  item['icon'] = 'font-awesome:cogs'
  items.push(item)

  items
end

items = []

if ARGV.empty?
  items = doing_show('currently')
else
  doing = ARGV.join(' ').strip
  items = doing_parse(doing)
end

puts items.to_json
